/* Pavel Surynek, 2008 */
/* pavel.surynek@mff.cuni.cz */
/* mit.h */

/*
  An implementation of the algorithm for solving sliding box puzzles by
  Kornhauser, Miler, and Spirakis.
 */

#ifndef __MIT_H__
#define __MIT_H__


#include <vector>
#include <list>

#include "bibox.h"

void save_fine_solution(FILE *f, const FINE_SOLUTION &fine_solution);
void load_fine_solution(FILE *f, FINE_SOLUTION &fine_solution);

void print_fine_solution(const FINE_SOLUTION &fine_solution);

void cycle_1_rotate_L(SLIDER *slider, FINE_SOLUTION &solution);
void cycle_1_rotate_R(SLIDER *slider, FINE_SOLUTION &solution);
void cycle_2_rotate_L(SLIDER *slider, FINE_SOLUTION &solution);
void cycle_2_rotate_R(SLIDER *slider, FINE_SOLUTION &solution);
void cycle_3_rotate_L(SLIDER *slider, FINE_SOLUTION &solution);
void cycle_3_rotate_R(SLIDER *slider, FINE_SOLUTION &solution);

int find_robot(SLIDER *slider, int r);
int find_robot_in_cycle_1(SLIDER *slider, int r);
int find_robot_in_cycle_2(SLIDER *slider, int r);
int find_robot_in_cycle_3(SLIDER *slider, int r);

void move_4_transitively_cycle_1(SLIDER *slider, int *robots, int *positions, FINE_SOLUTION &solution);
void move_4_transitively_cycle_1_half(SLIDER *slider, int *robots, FINE_SOLUTION &solution);

void execute_fine_solution(SLIDER *slider, const FINE_SOLUTION &solution);

bool generate_3_cycle(SLIDER *slider, int x, int y, int z, FINE_SOLUTION &solution);
void generate_3_cycle_CASE_A(SLIDER *slider, int x, int y, int z, FINE_SOLUTION &solution);
void generate_3_cycle_CASE_B(SLIDER *slider, int x, int y, int z, FINE_SOLUTION &solution);

int calculate_transpositions(SLIDER *slider);
int calculate_goal_transpositions(SLIDER *slider);

bool solve_slider_mit(SLIDER *slider, FINE_SOLUTION &solution);


#endif
